@@ -183,7 +183,8 @@ yamc::shared_lock<yamc::alternate::shared_mutex> lock(&mtx);
 				}
 				if (config_.objective != std::string("regression") && config_.objective != std::string("bernoulli_probit")
 					&& config_.objective != std::string("bernoulli_logit") && config_.objective != std::string("binary") 
-					&& config_.objective != std::string("poisson") && config_.objective != std::string("gamma")) {
+					&& config_.objective != std::string("poisson") && config_.objective != std::string("gamma") && 
+					config_.objective != std::string("negative_binomial")) {
 					Log::Fatal("GPBoost currently does not support 'objective = %s' ", config_.objective.c_str());
 				}
 				// Make sure that objective for boosting and likelihood for re_model match, otherwise change them accordingly
@@ -2704,6 +2705,8 @@ int GPB_CreateREModel(int32_t num_data,
 	int num_neighbors,
 	const char* vecchia_ordering,
 	int num_ind_points,
+	double cover_tree_radius,
+	const char* ind_points_selection,
 	const char* likelihood,
 	const char* matrix_inversion_method,
 	int seed,
@@ -2731,6 +2734,8 @@ int GPB_CreateREModel(int32_t num_data,
 		num_neighbors,
 		vecchia_ordering,
 		num_ind_points,
+		cover_tree_radius,
+		ind_points_selection,
 		likelihood,
 		matrix_inversion_method,
 		seed));
