@@ -266,6 +266,9 @@ internal::bfgs_impl(
     
         //ChangedForGPBoost
         //OPTIM_BFGS_TRACE(iter, grad_err, rel_sol_change, x, d, grad, s, y, W);
+        if (settings_inp) {
+            settings_inp->opt_iter = iter;
+        }
         if ((iter < 10 || (iter % 10 == 0 && iter < 100) || (iter % 100 == 0 && iter < 1000) ||
             (iter % 1000 == 0 && iter < 10000) || (iter % 10000 == 0)) && (iter != iter_max)) {
             Log::REDebug("GPModel parameter optimization iteration number %d", iter);
